'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.default = createAnewSelectorHOC;

var _reselect = require('reselect');

function createAnewSelectorHOC(anewStore, reduxStore, selectorName) {
    return function createAnewSelector() {
        for (var _len = arguments.length, payload = Array(_len), _key = 0; _key < _len; _key++) {
            payload[_key] = arguments[_key];
        }

        if (typeof payload[0] === 'string') {
            var propSelector = function propSelector() {
                var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
                    _ref$propName = _ref[propName],
                    propValue = _ref$propName === undefined ? defaultValue : _ref$propName;

                return propValue;
            };

            var propName = payload[0];
            var defaultValue = payload[1];

            reduxStore.getState[selectorName] = propSelector;
            anewStore.select[selectorName] = propSelector;

            return propSelector;
        } else if (payload.length === 1) {
            var selector = payload[0];

            if (typeof selector === 'function') {
                var simpleSelector = function simpleSelector() {
                    var simpleSelectorBinded = void 0;

                    if (reduxStore.anew.core) {
                        simpleSelectorBinded = function simpleSelectorBinded() {
                            for (var _len2 = arguments.length, payload = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
                                payload[_key2] = arguments[_key2];
                            }

                            return selector.apply(undefined, [anewStore.state, reduxStore.anew.core.getState()].concat(payload));
                        };
                    } else {
                        simpleSelectorBinded = function simpleSelectorBinded() {
                            for (var _len3 = arguments.length, payload = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
                                payload[_key3] = arguments[_key3];
                            }

                            return selector.apply(undefined, [anewStore.state].concat(payload));
                        };
                    }

                    reduxStore.getState[selectorName] = simpleSelectorBinded;
                    anewStore.select[selectorName] = simpleSelectorBinded;

                    return simpleSelectorBinded.apply(undefined, arguments);
                };

                return simpleSelector;
            }
        } else if (payload.length > 1) {
            var memoizedSelector = function memoizedSelector(state) {
                var memoizedSelectorBinded = void 0;

                if (reduxStore.anew.core) {
                    state = reduxStore.anew.core.getState();

                    memoizedSelectorBinded = function memoizedSelectorBinded() {
                        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : reduxStore.anew.core.getState();

                        return _selector(state);
                    };
                } else {
                    state = anewStore.state;

                    memoizedSelectorBinded = function memoizedSelectorBinded() {
                        var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : anewStore.state;

                        return _selector(state);
                    };
                }

                reduxStore.getState[selectorName] = memoizedSelectorBinded;
                anewStore.select[selectorName] = memoizedSelectorBinded;

                return memoizedSelectorBinded();
            };

            var _selector = _reselect.createSelector.apply(undefined, payload);

            return memoizedSelector;
        }
    };
}